export declare type CopyBufferData = {
    type: 'annotation' | 'empty';
    data?: {
        pageIndex: number;
        annotation: AnnotationBase;
    };
};
export declare type AnnotationStateModel = 'Marked' | 'Review';
export declare type AnnotationMarkedStateType = 'Marked' | 'Unmarked';
export declare type AnnotationReviewStateType = 'None' | 'Accepted' | 'Cancelled' | 'Completed' | 'Rejected';
export declare type AnnotationTypeName = ('Text' | 'Link' | 'FreeText' | 'Line' | 'Square' | 'Circle' | 'Polygon' | 'PolyLine' | 'Highlight' | 'Underline' | 'Squiggly' | 'Strikeout' | 'Stamp' | 'Caret' | 'Ink' | 'Popup' | 'FileAttachment' | 'Sound' | 'Movie' | 'Widget' | 'Screen' | 'PrinterMark' | 'TrapNet' | 'WaterMark' | 'Redact' | 'Signature' | 'ThreadBead' | 'RadioButton' | 'Checkbox' | 'PushButton' | 'Choice' | 'TextWidget' | 'Redact');
export declare type WidgetFieldTypeName = ('Tx' | 'Btn' | 'Ch');
export declare type LineEndStyle = ("Square" | "Circle" | "Diamond" | "OpenArrow" | "ClosedArrow" | "None" | "Butt" | "ROpenArrow" | "RClosedArrow" | "Slash");
export declare enum AnnotationTypeCode {
    TEXT = 1,
    LINK = 2,
    FREETEXT = 3,
    LINE = 4,
    SQUARE = 5,
    CIRCLE = 6,
    POLYGON = 7,
    POLYLINE = 8,
    HIGHLIGHT = 9,
    UNDERLINE = 10,
    SQUIGGLY = 11,
    STRIKEOUT = 12,
    STAMP = 13,
    CARET = 14,
    INK = 15,
    POPUP = 16,
    FILEATTACHMENT = 17,
    SOUND = 18,
    MOVIE = 19,
    WIDGET = 20,
    SCREEN = 21,
    PRINTERMARK = 22,
    TRAPNET = 23,
    WATERMARK = 24,
    THREED = 25,
    REDACT = 26,
    SIGNATURE = 27,
    THREADBEAD = 150
}
export declare enum TextAlignmentType {
    Left = 0,
    Center = 1,
    Right = 2
}
export declare function isFormFieldWidget(item: AnnotationBase): boolean;
export declare function isRadioButtonFieldFidget(item: AnnotationBase): boolean;
export declare function findAnnotationSubType(annotationType: AnnotationTypeCode, defaultValue: string | AnnotationTypeName): AnnotationTypeName;
export declare class AnnotationBase {
    id: string;
    annotationName: string;
    display?: 'visible' | 'hidden';
    orderIndex: number;
    annotationFlags: number;
    annotationType: AnnotationTypeCode;
    borderStyle?: {
        width: number;
        style: number;
        dashArray?: number[];
        horizontalCornerRadius: number;
        verticalCornerRadius: number;
    };
    appearanceColor?: Uint8ClampedArray | number[];
    color?: Uint8ClampedArray | number[] | null;
    contents?: string;
    textAlignment: TextAlignmentType;
    isRichContents: boolean;
    subtype: AnnotationTypeName;
    title: string;
    subject: string;
    rect: number[];
    useCustomAppearance: boolean;
    printableFlag: boolean;
    creationDate: string;
    modificationDate: string;
    popupId?: string;
    parentId?: string;
    parentAnnotation?: AnnotationBase;
    referenceType: 'R' | 'Group';
    referenceAnnotationId: string;
    irtAnnotations?: AnnotationBase[];
    state?: AnnotationMarkedStateType | AnnotationReviewStateType;
    stateModel?: AnnotationStateModel;
    invisibleFlag: boolean;
    __stateVersion: number;
    constructor();
}
export declare class PopupAnnotation extends AnnotationBase {
    open: boolean;
    constructor();
}
export declare class MarkupAnnotation extends AnnotationBase {
    hasPopup: boolean;
    popupId: string;
    constructor();
}
export declare class LineAnnotation extends MarkupAnnotation {
    lineCoordinates: number[];
    interiorColor?: Uint8ClampedArray | number[];
    constructor();
}
export declare class PolyLineAnnotation extends MarkupAnnotation {
    vertices: {
        x: number;
        y: number;
    }[];
    interiorColor?: Uint8ClampedArray | number[];
    constructor();
}
export declare class PolygonAnnotation extends PolyLineAnnotation {
    constructor();
}
export declare class CircleAnnotation extends MarkupAnnotation {
    cx: number;
    cy: number;
    rx: number;
    ry: number;
    interiorColor?: Uint8ClampedArray | number[];
    constructor();
}
export declare class SquareAnnotation extends MarkupAnnotation {
    interiorColor?: Uint8ClampedArray | number[];
    constructor();
}
export declare class RedactAnnotation extends MarkupAnnotation {
    markBorderColor?: Uint8ClampedArray | number[];
    markFillColor?: Uint8ClampedArray | number[];
    overlayFillColor?: Uint8ClampedArray | number[];
    overlayText?: string;
    overlayTextJustification?: TextAlignmentType;
    redactApplied: boolean;
    constructor();
}
export declare class InkAnnotation extends MarkupAnnotation {
    inkLists: {
        x: number;
        y: number;
    }[][];
    constructor();
}
export declare class LinkAnnotation extends MarkupAnnotation {
    url: string;
    dest: string;
    action: string;
    jsAction: string;
    newWindow: boolean;
    constructor();
}
export declare class TextAnnotation extends MarkupAnnotation {
    static DEFAULT_WIDTH: number;
    static DEFAULT_HEIGHT: number;
    open: boolean;
    name: 'Comment' | 'Key' | 'Note' | 'Help' | 'NewParagraph' | 'Paragraph' | 'Insert';
    state?: AnnotationMarkedStateType | AnnotationReviewStateType;
    stateModel?: AnnotationStateModel;
    displayState: AnnotationMarkedStateType | AnnotationReviewStateType;
    isRichContents: boolean;
    constructor();
}
export declare class FreeTextAnnotation extends MarkupAnnotation {
    calloutLine: number[];
    calloutLineEnd: LineEndStyle;
    isRichContents: boolean;
    constructor();
}
export declare class FileAttachmentAnnotation extends MarkupAnnotation {
    file: {
        filename: string;
        content: any;
    };
    isNew: boolean;
    constructor();
}
export declare class SoundAnnotation extends MarkupAnnotation {
    name: 'Speaker' | 'Mic';
    soundBytes: Uint8Array;
    audioProperties: {
        numChannels: 1 | 2 | number;
        sampleRate: number;
        bytesPerSample: 2 | 8 | 32 | 128 | 256 | number;
        subchunk2Size: 538100 | number;
    };
    playerOptions: {
        encoding: "16bitInt";
        channels: 1 | 2 | number;
        sampleRate: 44100 | number;
    };
    updatedAudioProps: {
        updatedSoundFile: Uint8Array | null;
        soundBytes: Uint8Array;
        audioProperties: {
            numChannels: number;
            sampleRate: number;
            bytesPerSample: number;
            subchunk2Size: number;
        };
        playerOptions: {
            encoding: "16bitInt";
            channels: number;
            sampleRate: number;
        };
    };
    constructor();
}
export declare class WidgetAnnotation extends AnnotationBase {
    fieldType: WidgetFieldTypeName;
    fieldName: string;
    fieldValue: string | string[];
    fontSize: number;
    readOnly: boolean;
    constructor();
}
export declare class TextWidget extends WidgetAnnotation {
    multiLine: boolean;
    comb: boolean;
    maxLen: number;
    hasPasswordFlag: boolean;
    backgroundColor: string | number[] | Uint8ClampedArray;
    borderColor: string;
    constructor();
}
export declare class ButtonWidget extends WidgetAnnotation {
    checkBox: boolean;
    radioButton: boolean;
    pushButton: boolean;
    SubmitForm: boolean;
    ResetForm: boolean;
    buttonValue: string;
    constructor();
}
export declare class ChoiceWidget extends WidgetAnnotation {
    combo: boolean;
    multiSelect: boolean;
    options: {
        displayValue: string;
        exportValue: string;
    }[];
    constructor();
}
