import { Component } from 'react';
import { AnnotationsModel, LayoutMode } from './types';
import { AnnotationBase } from './AnnotationTypes';
import { PropertyDescriptor } from '@grapecity/core-ui-react';
import PdfReportPlugin from '../plugin';
import { i18n } from 'i18next';
export declare type GcPdfViewerEditorsLocalization = {
    plainTextEditor: {
        emptyPlaceholder: string;
        multipleValuesPlaceholder: string;
    };
    collectionEditor: {
        closeBtnTitle: string;
        showBtnTitle: string;
        addBtnText: string;
        addBtnTitle: string;
        empty: string;
        items: string;
    };
    boolEditor: {
        textTrue: string;
        textFalse: string;
    };
    textAreaEditor?: {
        typeTextHere: string;
        cancelBtn: string;
        okBtn: string;
        cancelBtnTitle: string;
        okBtnTitle: string;
        editBtn: string;
    };
};
export declare type AnnotationProps = {
    coordinatesOrigin: 'TopLeft' | 'BottomLeft';
    plugin: PdfReportPlugin;
    navigatePage: (pageIndex: number) => void;
    navigateAnnotation: (pageIndex: number, annotation: AnnotationBase, params?: {
        preserveExpanded?: boolean;
        preserveFocused?: boolean;
        toggle?: boolean;
    }) => void;
    addAnnotation: (pageIndex: number, annotation: AnnotationBase) => Promise<{
        pageIndex: number;
        annotation: AnnotationBase;
    }>;
    updateAnnotation: (pageIndex: number, annotation: AnnotationBase, skipExpand: boolean) => Promise<{
        pageIndex: number;
        annotation: AnnotationBase;
    }>;
    updateAnnotations: (pageIndex: number, annotations: AnnotationBase[]) => Promise<{
        pageIndex: number;
        annotations: AnnotationBase[];
    }>;
    removeAnnotation: (pageIndex: number, id: string) => void;
};
export declare class Annotations extends Component<AnnotationProps, AnnotationsModel> {
    private static _pendingScrollAnnotationId;
    private static _pendingScrollAnnotationTime;
    _outerElement: HTMLElement;
    protected in17n: i18n;
    private _focusedEdtorElement;
    private _needRememberFocusedElement;
    private _onWindowMouseMoveHandler;
    private _onWindowMouseUpHandler;
    constructor(props: any, context: any);
    componentWillUpdate(): void;
    componentDidUpdate(): void;
    render(): JSX.Element;
    protected _renderAnnotations(pageIndex: number, pageAnnotations: AnnotationBase[], expandedPageIndex: number, expandedAnnotationIds: any, selectedAnnotationId: string, layoutMode: LayoutMode): JSX.Element | null;
    _renderPropertiesPanel(pageIndex: number, node: AnnotationBase, nodes: AnnotationBase[], isSelectedAnnotation: boolean, annotationButton: JSX.Element, deleteBtn: JSX.Element, cloneBtn: JSX.Element): JSX.Element;
    setProperty(pageIndex: number, node: AnnotationBase, descriptor: PropertyDescriptor, value: any): Promise<void>;
    private _switchToCustomAppearanceIfNeeded;
    _updateWithPopupIfAny(pageIndex: number, node: AnnotationBase, descriptor: PropertyDescriptor, updatedValue: any): Promise<boolean>;
    protected _onCloneClick: (event: Event | undefined, pageIndex: number, node: AnnotationBase) => () => boolean;
    protected _onDeleteClick: (event: any, pageIndex: number, node: AnnotationBase) => () => boolean;
    protected _onResetRedactClick: (event: Event | undefined, pageIndex: number, node: AnnotationBase) => () => boolean;
    protected _onApplyRedactClick: (event: Event | undefined, pageIndex: number, node: AnnotationBase) => () => boolean;
    protected _onPageButtonClick: (pageIndex: number) => () => void;
    protected _onPageChevronButtonClick: (pageIndex: number) => () => void;
    protected _onPropertiesPanelClick: (pageIndex: number, node: AnnotationBase) => () => void;
    protected _onAnnotationClick: (pageIndex: number, node: AnnotationBase) => () => void;
    protected _onAnnotationChevronClick: (pageIndex: number, node: AnnotationBase) => () => void;
    protected static _dragStartInfo: DragInfo;
    private _onItemButtonMouseDown;
    protected _onAnnotationsOuterMouseMove(e: MouseEvent): void;
    private _removeDragSourceClass;
    private _onAnnotationsOuterMouseUp;
    private _getDragElementParentContainer;
    ensureHighlightClass(): void;
    private _unbindDragEvents;
    private _resetDragPositions;
    private _consumedClickTime;
    consumeClickEvent(): void;
    get isClickEventConsumed(): boolean;
    private _moveButtonsDown;
    static setScrollAnnotationIntoViewOnRender(id: string): void;
    private _getPendingScrollAnnotationId;
    private _prepareAnnotationProperties;
    protected _getAnnotationIconKey(node: AnnotationBase, redactApplied?: boolean): {
        annotationIconKey: string;
        defaultAnnotationIconKey: string;
    };
    protected _getAnnotationButtonTitle(node: AnnotationBase): string;
    private _getAnnotationButtonLabelInternal;
    protected _getAnnotationButtonLabel(node: AnnotationBase): string;
    private _updateAnnotationContentPosition;
    private _rememberFocusedElement;
    private _restoreFocusedElement;
    private _forgetFocusedElement;
}
declare class DragInfo {
    owner: Annotations;
    srcOrderIndex: number;
    pageIndex: number;
    node: AnnotationBase;
    nodes: AnnotationBase[];
    private _startElemY?;
    constructor(owner: Annotations, srcOrderIndex: number, pageIndex: number, node: AnnotationBase, nodes: AnnotationBase[]);
    get elem(): HTMLElement;
    get elemY(): number;
    startDragging(): void;
    finish(): void;
    destElem?: HTMLElement;
    pageY?: number;
    dragging: boolean;
}
export {};
