export declare type DocumentInfo = {
    title: string;
    creator: string;
    producer: string;
    author: string;
    subject: string;
    keywords: string;
    creationDate?: string;
    modifyDate?: string;
};
export declare type OpenedDocumentInfo = {
    docId: string;
    pagesCount: number;
    defaultViewPortSize: {
        w: number;
        h: number;
    };
    info: DocumentInfo;
};
export declare type DocumentModification = {
    renderInteractiveForms?: boolean;
    rotation?: number;
    formData?: any;
    annotationsData?: {
        newAnnotations: any[];
        updatedAnnotations: any[];
        removedAnnotations: any[];
    };
};
export declare type SignatureInfo = {
    ContactInfo: string;
    Location: string;
    SignerName: string;
    Reason: string;
    SignatureDigestAlgorithm: 'SHA1' | 'SHA256' | 'SHA384' | 'SHA512' | 'PKCS7SHA1';
    SignatureFormat: 'PKCS7Detached' | 'PKCS7SHA1';
    TimeStamp: {
        ServerUrl?: string;
        UserName?: string;
        Password?: string;
    };
    SignatureField: string;
};
export interface ISupportApi {
    docId: string;
    docInfo: OpenedDocumentInfo;
    close(): Promise<string>;
    openUrl(url: string): Promise<OpenedDocumentInfo>;
    openBinary(data: any): Promise<OpenedDocumentInfo>;
    getDownloadUrl(filename: string): string;
    modify(documentModification: DocumentModification): Promise<boolean>;
    sign(signatureInfo: SignatureInfo): Promise<boolean>;
    verifySignature(fieldName: string): Promise<boolean>;
    serverVersion(): Promise<string>;
    ping(): Promise<string>;
}
